/*

    Till Harbaum

    t.harbaum@tu-bs.de
    http://www.ibr.cs.tu-bs.de/
*/


#include <Pilot.h>
#include <System/SysEvtMgr.h>

#include "accel_testRsc.h"
#include "tilt.h"

/////////////////////////
// Function Prototypes
/////////////////////////

static Boolean StartApplication(void);
static void    StopApplication(void);
static Boolean MainFormHandleEvent(EventPtr event);
static Boolean ApplicationHandleEvent(EventPtr event);
static void    EventLoop(void);

char tmp[32];
WinHandle draw_win;

/* reference for tilt sensor driver */
UInt TiltRef=0;

static Boolean StartApplication(void)
{
  short err;

  /* try to open tilt sensor lib */
  err = SysLibFind("Tilt Sensor Lib", &TiltRef);
  if (err) err = SysLibLoad('libr', 'Tilt', &TiltRef);
    
  if(!err) {
    /* try to initialize sensor */
    err = TiltLibOpen(TiltRef);
    if(err != 0) {
      if(err == TILT_WRONG_OS)  
	FrmCustomAlert(alt_err, "Wrong tilt sensor support installed.",0,0);

      else if(err == TILT_NO_SENSOR) 
	FrmCustomAlert(alt_err, "Tilt sensor hardware not found.",0,0);

      else if(err == TILT_HW_BUSY) 
	FrmCustomAlert(alt_err, "Tilt sensor interface is busy.",0,0);

      else
	FrmCustomAlert(alt_err, "Unknown error during tilt sensor detection.",0,0);

      return false;
    }
  } else {
    FrmCustomAlert(alt_err, "Unable to open tilt sensor driver.",0,0);
    return false;
  }

  /* calibrate sensor */
  TiltLibZero(TiltRef);

  FrmGotoForm(frm_Main);
  return true;
}

static void StopApplication(void) 
{
  UInt numapps;

  if(TiltRef != 0) {
    /* close sensor lib */
    TiltLibClose(TiltRef, &numapps);

    /* Check for errors in the Close() routine */
    if (numapps == 0) {
      SysLibRemove(TiltRef);
    }
  }
}

static Boolean MainFormHandleEvent(EventPtr event)
{
  Boolean handled = false;
  int i;

  switch (event->eType) {
  case frmOpenEvent:
    FrmDrawForm(FrmGetActiveForm());
    draw_win=WinGetDrawWindow();

    handled = true;
    break;
  }

  return(handled);
}

/////////////////////////
// ApplicationHandleEvent
/////////////////////////

static Boolean ApplicationHandleEvent(EventPtr event)
{
  FormPtr form;
  Word    formId;
  Boolean handled = false;
  
  if (event->eType == frmLoadEvent) {
    formId = event->data.frmLoad.formID;
    form = FrmInitForm(formId);
    FrmSetActiveForm(form);
    
    switch (formId) {
    case frm_Main:
      FrmSetEventHandler(form, MainFormHandleEvent);
      break;
    }
    handled = true;
  }

  return handled;
}

/////////////////////////
// EventLoop
/////////////////////////

#define TICKSWAIT 10

static void EventLoop(void)
{
  EventType event;
  int  error, count=0, x, y, last_y=0, last_x=0;
  Word  ticks_wait;
  long  last_ticks=0;

  /* initialize Timer */
  ticks_wait = SysTicksPerSecond()/10;

  do {
    EvtGetEvent(&event, ticks_wait);

    if (SysHandleEvent(&event))
      continue;
    if (ApplicationHandleEvent(&event))
      continue;

    /* do some kind of timer event */
    if(TimGetTicks()>last_ticks) {

      TiltLibGet(TiltRef, (Word *)&x, (Word *)&y);

      /* scroll area */
      if(draw_win==WinGetActiveWindow()) {
	RectangleType rect; 
	rect.topLeft.x = 1;   rect.topLeft.y = 15;
	rect.extent.x  = 159; rect.extent.y  = 160-15-10;
	WinCopyRectangle(draw_win, draw_win, &rect, 0, 15, scrCopy);
      }

      /* erase rightmost column */
      WinEraseLine(159, 15, 159, 149);

      StrPrintF(tmp, "%d     ", x);
      WinDrawChars(tmp, StrLen(tmp), 0, 150);

      StrPrintF(tmp, "%d     ", y);
      WinDrawChars(tmp, StrLen(tmp), 20, 150);

      /* draw graphic bars */
      if(y < -30) y = -30; 
      if(y >  30) y =  30;

      if(x < -30) x = -30; 
      if(x >  30) x =  30;

      /* draw new part of bar */
      WinDrawLine(158,  50+last_x, 159,  50+x); 
      WinDrawLine(158, 110+last_y, 159, 110+y); 

      last_x = x;
      last_y = y;

      last_ticks=TimGetTicks()+ticks_wait;
    }

    /* prevent from auto power off */
    EvtResetAutoOffTimer();

    FrmDispatchEvent(&event);
  } while (event.eType != appStopEvent);
}

/////////////////////////
// PilotMain
/////////////////////////

DWord PilotMain(Word cmd, Ptr cmdBPB, Word launchFlags)
{
  if (cmd == sysAppLaunchCmdNormalLaunch)
  {
    if(StartApplication()) 
      EventLoop();

    StopApplication();
  }
  return(0);
}













